@@ -18,15 +18,9 @@
 #ifndef CONNECTION_H
 #define CONNECTION_H
 
+#include "diatypes.h"
 #include "object.h"
-
-typedef struct _Connection Connection;
-typedef struct _ConnectionBBExtras ConnectionBBExtras;
-
-struct _ConnectionBBExtras {
-  real start_long, start_trans;
-  real end_long, end_trans;
-};
+#include "boundingbox.h"
 
 /* This is a subclass of Object used to help implementing objects
  * of a line-like type. */
@@ -36,7 +30,7 @@ struct _Connection {
   
   Point endpoints[2];
   Handle endpoint_handles[2];
-  ConnectionBBExtras extra_spacing;
+  LineBBExtras extra_spacing;
 };
 
 void connection_update_handles(Connection *conn);
@@ -47,8 +41,10 @@ void connection_destroy(Connection *conn);
 void connection_copy(Connection *from, Connection *to);
 void connection_save(Connection *conn, ObjectNode obj_node);
 void connection_load(Connection *conn, ObjectNode obj_node);
-void connection_move_handle(Connection *conn, HandleId id,
-			    Point *to, HandleMoveReason reason);
+ObjectChange* connection_move_handle(Connection *conn, HandleId id,
+				     Point *to, ConnectionPoint* cp,
+				     HandleMoveReason reason, 
+				     ModifierKeys modifiers);
 
 /* base property stuff... */
 #define CONNECTION_COMMON_PROPERTIES \
