@@ -15,21 +15,196 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
+#include <config.h>
+
 #include <glib.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <locale.h>
+#include <math.h>
+#include <fcntl.h>
 
-#include <parser.h>
-#if defined(LIBXML_VERSION) && LIBXML_VERSION >= 20000
-#define childs children
+#include <libxml/parser.h>
+#include <libxml/parserInternals.h>
+#include <libxml/xmlmemory.h>
+
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>
 #endif
 
+#include <zlib.h>
+
+#include "intl.h"
 #include "utils.h"
+#include "dia_xml_libxml.h"
 #include "dia_xml.h"
+#include "geometry.h"		/* For isinf() on Solaris */
 #include "message.h"
 
+#ifdef G_OS_WIN32
+#include <io.h> /* write, close */
+#endif
+
+#ifdef G_OS_WIN32 /* apparently _MSC_VER and mingw */
+#include <float.h>
+#define isinf(a) (!_finite(a))
+#endif
+
+#define BUFLEN 1024
+
+
+static const gchar *
+xml_file_check_encoding(const gchar *filename, const gchar *default_enc)
+{
+  /* If all files produced by dia were good XML files, we wouldn't have to do 
+     this little gymnastic. Alas, during the libxml1 days, we were outputting 
+     files with no encoding specification (which means UTF-8 if we're in an
+     asciish encoding) and strings encoded in local charset (so, we wrote
+     broken files). 
+
+     The following logic finds if we have a broken file, and attempts to fix 
+     it if it's possible. If the file is correct or is unrecognisable, we pass
+     it untouched to libxml2.
+  */
+
+  gzFile zf = gzopen(filename,"rb");  
+  gchar *buf;
+  gchar *p,*pmax;
+  int len;
+  gchar *tmp,*res;
+  int uf;
+  gboolean well_formed_utf8;
+
+  static char magic_xml[] = 
+  {0x3c,0x3f,0x78,0x6d,0x6c,0x00}; /* "<?xml" in ASCII */
+
+  if (!zf) {
+    /*    message_error(_("The file %s can not be opened for reading"),filename); */ 
+    /* XXX perhaps we can just chicken out to libxml ? -- CC */ 
+    return NULL;
+  }
+  p = buf = g_malloc0(BUFLEN);  
+  len = gzread(zf,buf,BUFLEN);
+  pmax = p + len;
+
+  /* first, we expect the magic <?xml string */
+  if ((0 != strncmp(p,magic_xml,5)) || (len < 5)) {
+    gzclose(zf);
+    return filename; /* let libxml figure out what this is. */
+  }
+  /* now, we're sure we have some asciish XML file. */
+  p += 5;
+  while (((*p == 0x20)||(*p == 0x09)||(*p == 0x0d)||(*p == 0x0a))
+         && (p<pmax)) p++;
+  if (p>=pmax) { /* whoops ? */
+    gzclose(zf);
+    return filename;
+  }
+  if (0 != strncmp(p,"version=\"",9)) {
+    gzclose(zf); /* chicken out. */
+    return filename;
+  }
+  p += 9;
+  /* The header is rather well formed. */
+  if (p>=pmax) { /* whoops ? */
+    gzclose(zf);
+    return filename;
+  }
+  while ((*p != '"') && (p < pmax)) p++;
+  p++;
+  while (((*p == 0x20)||(*p == 0x09)||(*p == 0x0d)||(*p == 0x0a))
+         && (p<pmax)) p++;
+  if (p>=pmax) { /* whoops ? */
+    gzclose(zf);
+    return filename;
+  }
+  if (0 == strncmp(p,"encoding=\"",10)) {
+    gzclose(zf); /* this file has an encoding string. Good. */
+    return filename;
+  }
+  /* now let's read the whole file, to see if there are offending bits.
+   * We can call it well formed UTF-8 if the highest isn't used
+   */
+  well_formed_utf8 = TRUE;
+  do {
+    int i;
+    for (i = 0; i < len; i++)
+      if (buf[i] & 0x80 || buf[i] == '&')
+        well_formed_utf8 = FALSE;
+    len = gzread(zf,buf,BUFLEN);
+  } while (len > 0 && well_formed_utf8);
+  if (well_formed_utf8) {
+    gzclose(zf); /* this file is utf-8 compatible  */
+    return filename;
+  } else {
+    gzclose(zf); /* poor man's fseek */
+    zf = gzopen(filename,"rb"); 
+    len = gzread(zf,buf,BUFLEN);
+  }
+
+  if (0 != strcmp(default_enc,"UTF-8")) {
+    message_warning(_("The file %s has no encoding specification;\n"
+                      "assuming it is encoded in %s"),filename,default_enc);
+  } else {
+    gzclose(zf); /* we apply the standard here. */
+    return filename;
+  }
+
+  tmp = getenv("TMP"); 
+  if (!tmp) tmp = getenv("TEMP");
+  if (!tmp) tmp = "/tmp";
+
+  res = g_strconcat(tmp,G_DIR_SEPARATOR_S,"dia-xml-fix-encodingXXXXXX",NULL);
+  uf = g_mkstemp(res);
+  write(uf,buf,p-buf);
+  write(uf," encoding=\"",11);
+  write(uf,default_enc,strlen(default_enc));
+  write(uf,"\" ",2);
+  write(uf,p,pmax - p);
+
+  while (1) {
+    len = gzread(zf,buf,BUFLEN);
+    if (len <= 0) break;
+    write(uf,buf,len);
+  }
+  gzclose(zf);
+  close(uf);
+  return res; /* caller frees the name and unlinks the file. */
+}
+
+
+
+xmlDocPtr
+xmlDiaParseFile(const char *filename) {
+  G_CONST_RETURN char *local_charset = NULL;
+  
+  if (   !g_get_charset(&local_charset)
+      && local_charset) {
+    /* we're not in an UTF-8 environment. */ 
+    const gchar *fname = xml_file_check_encoding(filename,local_charset);
+    if (fname != filename) {
+      /* We've got a corrected file to parse. */
+      xmlDocPtr ret = xmlDoParseFile(fname);
+      unlink(fname);
+      /* printf("has read %s instead of %s\n",fname,filename); */
+      g_free((void *)fname);
+      return ret;
+    } else {
+      /* the XML file is good. libxml is "old enough" to handle it correctly.
+       */
+      return xmlDoParseFile(filename);        
+    }
+  } else {
+    return xmlDoParseFile(filename);
+  }
+}
+
+xmlDocPtr
+xmlDoParseFile(const char *filename) {
+  return xmlParseFile(filename);
+}
+
 AttributeNode
 object_find_attribute(ObjectNode obj_node,
 		      const char *attrname)
@@ -37,15 +212,23 @@ object_find_attribute(ObjectNode obj_node,
   AttributeNode attr;
   char *name;
 
-  attr =  obj_node->childs;
+  while (obj_node && xmlIsBlankNode(obj_node)) 
+    obj_node = obj_node->next;
+  if (!obj_node) return NULL;
+
+  attr =  obj_node->xmlChildrenNode;
   while (attr != NULL) {
+    if (xmlIsBlankNode(attr)) {
+      attr = attr->next;
+      continue;
+    }
 
     name = xmlGetProp(attr, "name");
     if ( (name!=NULL) && (strcmp(name, attrname)==0) ) {
-      free(name);
+      xmlFree(name);
       return attr;
     }
-    if (name) free(name);
+    if (name) xmlFree(name);
     
     attr = attr->next;
   }
@@ -59,15 +242,23 @@ composite_find_attribute(DataNode composite_node,
   AttributeNode attr;
   char *name;
 
-  attr =  composite_node->childs;
+  while (composite_node && xmlIsBlankNode(composite_node)) 
+    composite_node = composite_node->next;
+  if (!composite_node) return NULL;
+
+  attr =  composite_node->xmlChildrenNode;
   while (attr != NULL) {
+    if (xmlIsBlankNode(attr)) {
+      attr = attr->next;
+      continue;
+    }
 
     name = xmlGetProp(attr, "name");
     if ( (name!=NULL) && (strcmp(name, attrname)==0) ) {
-      free(name);
+      xmlFree(name);
       return attr;
     }
-    if (name) free(name);
+    if (name) xmlFree(name);
     
     attr = attr->next;
   }
@@ -80,8 +271,12 @@ attribute_num_data(AttributeNode attribute)
   xmlNode *data;
   int nr=0;
 
-  data =  attribute->childs;
+  data =  attribute ? attribute->xmlChildrenNode : NULL;
   while (data != NULL) {
+    if (xmlIsBlankNode(data)) {
+      data = data->next;
+      continue;
+    }
     nr++;
     data = data->next;
   }
@@ -91,13 +286,20 @@ attribute_num_data(AttributeNode attribute)
 DataNode
 attribute_first_data(AttributeNode attribute)
 {
-  return (DataNode) attribute->childs;
+  xmlNode *data = attribute ? attribute->xmlChildrenNode : NULL;
+  while (data && xmlIsBlankNode(data)) data = data->next;
+  return (DataNode) data;
 }
 
 DataNode
 data_next(DataNode data)
 {
-  return (DataNode) data->next;
+  
+  if (data) { 
+    data = data->next;
+    while (data && xmlIsBlankNode(data)) data = data->next;
+  }
+  return (DataNode) data;
 }
 
 DataType
@@ -105,7 +307,7 @@ data_type(DataNode data)
 {
   const char *name;
 
-  name = data->name;
+  name = data ? (const char *)data->name : (const char *)"";
   if (strcmp(name, "composite")==0) {
     return DATATYPE_COMPOSITE;
   } else if (strcmp(name, "int")==0) {
@@ -146,7 +348,7 @@ data_int(DataNode data)
 
   val = xmlGetProp(data, "val");
   res = atoi(val);
-  if (val) free(val);
+  if (val) xmlFree(val);
   
   return res;
 }
@@ -163,7 +365,7 @@ int data_enum(DataNode data)
 
   val = xmlGetProp(data, "val");
   res = atoi(val);
-  if (val) free(val);
+  if (val) xmlFree(val);
   
   return res;
 }
@@ -173,15 +375,18 @@ data_real(DataNode data)
 {
   char *val;
   real res;
-  
+  char *old_locale;
+
   if (data_type(data)!=DATATYPE_REAL) {
     message_error("Taking real value of non-real node.");
     return 0;
   }
 
   val = xmlGetProp(data, "val");
-  res = g_strtod(val, NULL);
-  if (val) free(val);
+  old_locale = setlocale(LC_NUMERIC, "C");
+  res = strtod(val, NULL);
+  setlocale(LC_NUMERIC, old_locale);
+  if (val) xmlFree(val);
   
   return res;
 }
@@ -204,7 +409,7 @@ data_boolean(DataNode data)
   else 
     res = FALSE;
 
-  if (val) free(val);
+  if (val) xmlFree(val);
 
   return res;
 }
@@ -242,7 +447,7 @@ data_color(DataNode data, Color *col)
     b = hex_digit(val[5])*16 + hex_digit(val[6]);
   }
 
-  if (val) free(val);
+  if (val) xmlFree(val);
   
   col->red = ((float)r)/255.0;
   col->green = ((float)g)/255.0;
@@ -254,25 +459,43 @@ data_point(DataNode data, Point *point)
 {
   char *val;
   char *str;
-  
+  char *old_locale;
+  real ax,ay;
+
   if (data_type(data)!=DATATYPE_POINT) {
-    message_error("Taking point value of non-point node.");
+    message_error(_("Taking point value of non-point node."));
     return;
   }
   
   val = xmlGetProp(data, "val");
-  point->x = g_strtod(val, &str);
+  old_locale = setlocale(LC_NUMERIC, "C");
+  point->x = strtod(val, &str);
+  ax = fabs(point->x);
+  if ((ax > 1e9) || ((ax < 1e-9) && (ax != 0.0)) || isnan(ax) || isinf(ax)) {
+    /* there is no provision to keep values larger when saving, 
+     * so do this 'reduction' silent */
+    if (!(ax < 1e-9)) 
+      g_warning(_("Incorrect x Point value \"%s\" %f; discarding it."),val,point->x);
+    point->x = 0.0;
+  }
   while ((*str != ',') && (*str!=0))
     str++;
   if (*str==0){
+    setlocale(LC_NUMERIC, old_locale);
     point->y = 0.0;
-    message_error("Error parsing point.");
-    free(val);
+    g_error(_("Error parsing point."));
+    xmlFree(val);
     return;
   }
-    
-  point->y = g_strtod(str+1, NULL);
-  free(val);
+  point->y = strtod(str+1, NULL);
+  ay = fabs(point->y);
+  if ((ay > 1e9) || ((ay < 1e-9) && (ay != 0.0)) || isnan(ay) || isinf(ay)) {
+    if (!(ay < 1e-9)) /* don't bother with useless warnings (see above) */
+      g_warning(_("Incorrect y Point value \"%s\" %f; discarding it."),str+1,point->y);
+    point->y = 0.0;
+  }
+  setlocale(LC_NUMERIC, old_locale);
+  xmlFree(val);
 }
 
 void
@@ -280,6 +503,7 @@ data_rectangle(DataNode data, Rectangle *rect)
 {
   char *val;
   char *str;
+  char *old_locale;
   
   if (data_type(data)!=DATATYPE_RECTANGLE) {
     message_error("Taking rectangle value of non-rectangle node.");
@@ -288,49 +512,57 @@ data_rectangle(DataNode data, Rectangle *rect)
   
   val = xmlGetProp(data, "val");
   
-  rect->left = g_strtod(val, &str);
+  old_locale = setlocale(LC_NUMERIC, "C");
+  rect->left = strtod(val, &str);
+  setlocale(LC_NUMERIC, old_locale);
   
   while ((*str != ',') && (*str!=0))
     str++;
 
   if (*str==0){
     message_error("Error parsing rectangle.");
-    free(val);
+    xmlFree(val);
     return;
   }
     
-  rect->top = g_strtod(str+1, &str);
+  old_locale = setlocale(LC_NUMERIC, "C");
+  rect->top = strtod(str+1, &str);
+  setlocale(LC_NUMERIC, old_locale);
 
   while ((*str != ';') && (*str!=0))
     str++;
 
   if (*str==0){
     message_error("Error parsing rectangle.");
-    free(val);
+    xmlFree(val);
     return;
   }
 
-  rect->right = g_strtod(str+1, &str);
+  old_locale = setlocale(LC_NUMERIC, "C");
+  rect->right = strtod(str+1, &str);
+  setlocale(LC_NUMERIC, old_locale);
 
   while ((*str != ',') && (*str!=0))
     str++;
 
   if (*str==0){
     message_error("Error parsing rectangle.");
-    free(val);
+    xmlFree(val);
     return;
   }
 
-  rect->bottom = g_strtod(str+1, NULL);
+  old_locale = setlocale(LC_NUMERIC, "C");
+  rect->bottom = strtod(str+1, NULL);
+  setlocale(LC_NUMERIC, old_locale);
   
-  free(val);
+  xmlFree(val);
 }
 
-char *
+gchar *
 data_string(DataNode data)
 {
-  char *val;
-  char *str, *p;
+  gchar *val;
+  gchar *str, *p,*str2;
   int len;
   
   if (data_type(data)!=DATATYPE_STRING) {
@@ -340,8 +572,8 @@ data_string(DataNode data)
 
   val = xmlGetProp(data, "val");
   if (val != NULL) { /* Old kind of string. Left for backwards compatibility */
-    str  = g_malloc(sizeof(char)*(strlen(val)+1));
-    
+    str  = g_malloc(4 * (sizeof(char)*(strlen(val)+1))); /* extra room 
+                                                            for UTF8 */
     p = str;
     while (*val) {
       if (*val == '\\') {
@@ -368,13 +600,15 @@ data_string(DataNode data)
       val++;
     }
     *p = 0;
-    free(val);
-    return str;
+    xmlFree(val);
+    str2 = g_strdup(str);  /* to remove the extra space */
+    g_free(str);
+    return str2;
   }
 
-  if (data->childs!=NULL) {
-    p = xmlNodeListGetString(data->doc, data->childs, TRUE);
-
+  if (data->xmlChildrenNode!=NULL) {
+    p = xmlNodeListGetString(data->doc, data->xmlChildrenNode, TRUE);
+    
     if (*p!='#')
       message_error("Error in file, string not starting with #\n");
     
@@ -387,32 +621,44 @@ data_string(DataNode data)
 
     str[strlen(str)-1] = 0; /* Remove last '#' */
     
-    free(p);
-
     return str;
   }
     
   return NULL;
 }
 
-Font *
+DiaFont *
 data_font(DataNode data)
 {
-  char *name;
-  Font *font;
+  char *family;
+  DiaFont *font;
   
   if (data_type(data)!=DATATYPE_FONT) {
     message_error("Taking font value of non-font node.");
     return NULL;
   }
 
-  name = xmlGetProp(data, "name");
-  font = font_getfont(name);
-  if (name) free(name);
-  
+  family = xmlGetProp(data, "family");
+  /* always prefer the new format */
+  if (family) {
+    DiaFontStyle style;
+    char* style_name = xmlGetProp(data, "style");
+    style = style_name ? atoi(style_name) : 0;
+
+    font = dia_font_new (family, style, 1.0);
+    if (family) xmlFree(family);
+    if (style_name) xmlFree(style_name);
+  } else {
+    /* Legacy format support */
+    char *name = xmlGetProp(data, "name");
+    font = dia_font_new_from_legacy_name(name);
+    xmlFree(name);
+  }
   return font;
 }
 
+/* ***** Saving XML **** */
+
 AttributeNode
 new_attribute(ObjectNode obj_node,
 	      const char *attrname)
@@ -503,7 +749,7 @@ static void convert_to_hex(float x, char *str)
 }
 
 void
-data_add_color(AttributeNode attr, Color *col)
+data_add_color(AttributeNode attr, const Color *col)
 {
   char buffer[1+6+1];
   DataNode data_node;
@@ -519,7 +765,7 @@ data_add_color(AttributeNode attr, Color *col)
 }
 
 void
-data_add_point(AttributeNode attr, Point *point)
+data_add_point(AttributeNode attr, const Point *point)
 {
   DataNode data_node;
   char buffer[80+1]; /* Large enought? */
@@ -534,10 +780,10 @@ data_add_point(AttributeNode attr, Point *point)
 }
 
 void
-data_add_rectangle(AttributeNode attr, Rectangle *rect)
+data_add_rectangle(AttributeNode attr, const Rectangle *rect)
 {
   DataNode data_node;
-  char buffer[160+1]; /* Large enought? */
+  char buffer[160+1]; /* Large enough? */
   char *old_locale;
 
   old_locale = setlocale(LC_NUMERIC, "C");
@@ -551,102 +797,48 @@ data_add_rectangle(AttributeNode attr, Rectangle *rect)
 
 }
 
-static int
-escaped_str_len(char *str)
-{
-  int len;
-  char c;
-  
-  if (str==NULL)
-    return 0;
-  
-  len = 0;
-
-  while ((c=*str++) != 0) {
-    switch (c) {
-    case '&':
-      len += 5; /* "&amp;" */
-      break;
-    case '<':
-    case '>':
-      len += 4; /* "&lt;" or "&gt;" */
-      break;
-    default:
-      len++;
-    }
-  }
-  
-  return len;
-}
-
-static void
-escape_string(char *buffer, char *str)
-{
-  int len;
-  char c;
-  
-  *buffer = 0;
-  
-  if (str==NULL) 
-    return;
-  
-  len = 0;
-
-  while ((c=*str++) != 0) {
-    switch (c) {
-    case '&':
-      strcpy(buffer, "&amp;");
-      buffer += 5;
-      break;
-    case '<':
-      strcpy(buffer, "&lt;");
-      buffer += 4;
-      break;
-    case '>':
-      strcpy(buffer, "&gt;");
-      buffer += 4;
-     break;
-    default:
-      *buffer = c;
-      buffer++;
-    }
-  }
-  *buffer = 0;
-}
-
 void
-data_add_string(AttributeNode attr, char *str)
+data_add_string(AttributeNode attr, const char *str)
 {
-  DataNode data_node;
-  char *escaped_str;
-  int len;
+    DataNode data_node;
+    xmlChar *escaped_str;
+    xmlChar *sharped_str;
 
-  if (str==NULL) {
-    data_node = xmlNewChild(attr, NULL, "string", NULL);
-  } else {
-    len = 2+escaped_str_len(str);
-    escaped_str = g_malloc(len+1);
-    *escaped_str='#';
-    escape_string(escaped_str+1, str);
-    strcat(escaped_str, "#");
+    if (str==NULL) {
+        data_node = xmlNewChild(attr, NULL, "string", "##");
+        return;
+    } 
+
+    escaped_str = xmlEncodeEntitiesReentrant(attr->doc,str);
     
-    data_node = xmlNewChild(attr, NULL, "string", escaped_str);
+    sharped_str = g_strconcat("#", escaped_str, "#", NULL);
+
+    xmlFree(escaped_str);
+    
+    data_node = xmlNewChild(attr, NULL, "string", sharped_str);
   
-    g_free(escaped_str);
-  }
+    g_free(sharped_str);
 }
 
 void
-data_add_font(AttributeNode attr, Font *font)
+data_add_font(AttributeNode attr, const DiaFont *font)
 {
   DataNode data_node;
- 
+  DiaFontStyle style;
+  char buffer[20+1]; /* Enought for 64bit int + zero */
+
   data_node = xmlNewChild(attr, NULL, "font", NULL);
-  xmlSetProp(data_node, "name", font->name);
+  style = dia_font_get_style (font);
+  xmlSetProp(data_node, "family", dia_font_get_family(font));
+  g_snprintf(buffer, 20, "%d", dia_font_get_style(font));
+ 
+  xmlSetProp(data_node, "style", buffer);
+  /* Legacy support: don't crash older Dia on missing 'name' attribute */
+  xmlSetProp(data_node, "name", dia_font_get_legacy_name(font));
 }
 
 DataNode
-data_add_composite(AttributeNode attr, char *type) 
+data_add_composite(AttributeNode attr, const char *type) 
 {
   /* type can be NULL */
   DataNode data_node;
@@ -658,4 +850,31 @@ data_add_composite(AttributeNode attr, char *type)
   return data_node;
 }
 
+void 
+warn_about_broken_libxml1(void) 
+{
+  message_warning(_("Your local character set is UTF-8. Because of issues"
+                    " with libxml1 and the support of files generated by"
+                    " previous versions of dia, you will encounter "
+                    " problems. Please report to dia-list@gnome.org if you"
+                    " see this message."));
+}
 
+#define BUFSIZE 2048
+#define OVERRUN_SAFETY 16
+
+/* diarc option */
+int pretty_formated_xml = TRUE;
+
+int xmlDiaSaveFile(const char *filename,
+                   xmlDocPtr cur)
+{
+    int old = 0, ret;
+
+    if (pretty_formated_xml)
+        old = xmlKeepBlanksDefault (0);
+    ret = xmlSaveFormatFileEnc (filename,cur, "UTF-8", pretty_formated_xml ? 1 : 0);
+    if (pretty_formated_xml)
+        xmlKeepBlanksDefault (old);
+    return ret;
+}
