@@ -25,11 +25,13 @@
 
 #include <gdk/gdkkeysyms.h>
 
+#include "propinternals.h"
 #include "text.h"
 #include "message.h"
+#include "diarenderer.h"
 
 static int text_key_event(Focus *focus, guint keysym,
-			  char *str, int strlen,
+			  const gchar *str, int strlen,
 			  ObjectChange **change);
 
 enum change_type {
@@ -46,14 +48,16 @@ struct TextObjectChange {
 
   Text *text;
   enum change_type type;
-  char ch;
+  gunichar ch;
   int pos;
   int row;
-  char *str;
+  gchar *str;
 };
 
+#define CURSOR_HEIGHT_RATIO 20
+
 static ObjectChange *text_create_change(Text *text, enum change_type type,
-					char ch, int pos, int row);
+					gunichar ch, int pos, int row);
 
 static void
 calc_width(Text *text)
@@ -64,7 +68,7 @@ calc_width(Text *text)
   width = 0.0;
   for (i=0;i<text->numlines;i++) {
     text->row_width[i] =
-      font_string_width(text->line[i], text->font, text->height);
+      dia_font_string_width(text->line[i], text->font, text->height);
     width = MAX(width, text->row_width[i]);
   }
   
@@ -74,8 +78,16 @@ calc_width(Text *text)
 static void
 calc_ascent_descent(Text *text)
 {
-  text->ascent = font_ascent(text->font, text->height);
-  text->descent = font_descent(text->font, text->height);
+  real sig_a = 0.0,sig_d = 0.0;
+  guint i;
+    
+  for ( i = 0; i < text->numlines; i++) {
+      sig_a += dia_font_ascent(text->line[i], text->font, text->height);
+      sig_d += dia_font_descent(text->line[i], text->font, text->height);
+  }
+  
+  text->ascent = sig_a / (real)text->numlines;
+  text->descent = sig_d / (real)text->numlines;
 }
 
 static void
@@ -104,18 +116,18 @@ static void
 set_string(Text *text, const char *string)
 {
   int numlines, i;
-  int len;
+  int alloclen;
   const char *s,*s2;
   
   s = string;
   
   numlines = 1;
   if (s != NULL) 
-    while ( (s = strchr(s,'\n')) != NULL ) {
-	s++;
-	if ((*s) != 0) {
-	    numlines++;
-	}
+    while ( (s = g_utf8_strchr(s,-1,'\n')) != NULL ) {
+      s++;
+      if ((*s) != 0) {
+        numlines++;
+      }
     }
   text->numlines = numlines;
   text->line = g_malloc(sizeof(char *)*numlines);
@@ -134,17 +146,17 @@ set_string(Text *text, const char *string)
   }
 
   for (i=0;i<numlines;i++) {
-    s2 = strchr(s,'\n');
+    s2 = g_utf8_strchr(s,-1,'\n');
     if (s2==NULL) {
-      len = strlen(s);
+      alloclen = strlen(s);
     } else {
-      len = s2 - s;
+      alloclen = s2 - s;
     }
-    text->line[i] = (char *)g_malloc(len+1);
-    text->strlen[i] = len;
-    text->alloclen[i] = len+1;
-    strncpy(text->line[i], s, len);
-    text->line[i][len] = 0; /* end line with \0 */
+    text->line[i] = (char *)g_malloc(alloclen+1);
+    text->alloclen[i] = alloclen+1;
+    strncpy (text->line[i], s, alloclen);
+    text->line[i][alloclen] = 0; /* end line with \0 */
+    text->strlen[i] = g_utf8_strlen(text->line[i], -1);;
     s = s2+1;
   }
 
@@ -169,14 +181,14 @@ text_set_string(Text *text, const char *string)
 }
 
 Text *
-new_text(const char *string, Font *font, real height,
+new_text(const char *string, DiaFont *font, real height,
 	 Point *pos, Color *color, Alignment align)
 {
   Text *text;
 
   text = g_new(Text, 1);
 
-  text->font = font;
+  text->font = dia_font_ref(font);
   text->height = height;
 
   text->position = *pos;
@@ -213,13 +225,13 @@ text_copy(Text *text)
   copy->row_width = g_malloc(sizeof(real)*copy->numlines);
   
   for (i=0;i<text->numlines;i++) {
-    copy->line[i] = (char *)g_malloc(text->strlen[i]+1);
+    copy->line[i] = (char *)g_malloc(text->alloclen[i]+1);
     strcpy(copy->line[i], text->line[i]);
     copy->strlen[i] = text->strlen[i];
-    copy->alloclen[i] = text->strlen[i]+1;
+    copy->alloclen[i] = text->alloclen[i];
   }
   
-  copy->font = text->font;
+  copy->font = dia_font_ref(text->font);
   copy->height = text->height;
   copy->position = text->position;
   copy->color = text->color;
@@ -246,6 +258,7 @@ void
 text_destroy(Text *text)
 {
   free_string(text);
+  dia_font_unref(text->font);
   g_free(text);
 }
 
@@ -259,9 +272,10 @@ text_set_height(Text *text, real height)
 }
 
 void
-text_set_font(Text *text, Font *font)
+text_set_font(Text *text, DiaFont *font)
 {
-  text->font = font;
+  dia_font_unref(text->font);
+  text->font = dia_font_ref(text->font);
   
   calc_width(text);
   calc_ascent_descent(text);
@@ -288,6 +302,11 @@ text_set_alignment(Text *text, Alignment align)
 void
 text_calc_boundingbox(Text *text, Rectangle *box)
 {
+  calc_width(text);
+  calc_ascent_descent(text);
+
+  if (box == NULL) return; /* For those who just want the text info
+			      updated */
   box->left = text->position.x;
   switch (text->alignment) {
   case ALIGN_LEFT:
@@ -302,13 +321,26 @@ text_calc_boundingbox(Text *text, Rectangle *box)
 
   box->right = box->left + text->max_width;
   
-  if (text->cursor_pos == 0) {
-    box->left -= 0.05; /* Half the cursor width */
-  }
-
   box->top = text->position.y - text->ascent;
 
-  box->bottom = box->top + text->height*text->numlines;
+  box->bottom = box->top + text->height*text->numlines + text->descent;
+
+  if (text->focus.has_focus) {
+    real height = text->ascent + text->descent;
+    if (text->cursor_pos == 0) {
+      /* Half the cursor width */
+      box->left -= height/(CURSOR_HEIGHT_RATIO*2);
+    } else {
+      /* Half the cursor width. Assume that
+	 if it isn't at position zero, it might be 
+	 at the last position possible. */
+      box->right += height/(CURSOR_HEIGHT_RATIO*2);
+    }
+   
+    /* Account for the size of the cursor top and bottom */
+    box->top -= height/(CURSOR_HEIGHT_RATIO*2);
+    box->bottom += height/CURSOR_HEIGHT_RATIO;
+  }
 }
 
 
@@ -384,17 +416,17 @@ text_distance_from(Text *text, Point *point)
 }
 
 void
-text_draw(Text *text, Renderer *renderer)
+text_draw(Text *text, DiaRenderer *renderer)
 {
   Point pos;
   int i;
   
-  renderer->ops->set_font(renderer, text->font, text->height);
+  DIA_RENDERER_GET_CLASS(renderer)->set_font(renderer, text->font, text->height);
   
   pos = text->position;
   
   for (i=0;i<text->numlines;i++) {
-    renderer->ops->draw_string(renderer,
+    DIA_RENDERER_GET_CLASS(renderer)->draw_string(renderer,
 			       text->line[i],
 			       &pos, text->alignment,
 			       &text->color);
@@ -407,18 +439,17 @@ text_draw(Text *text, Renderer *renderer)
     real str_width_first;
     real str_width_whole;
     Point p1, p2;
-
+    real height = text->ascent+text->descent;
     curs_y = text->position.y - text->ascent + text->cursor_row*text->height; 
 
     str_width_first =
-      renderer->interactive_ops->get_text_width(renderer,
+      DIA_RENDERER_GET_CLASS(renderer)->get_text_width(renderer,
 						text->line[text->cursor_row],
 						text->cursor_pos);
     str_width_whole =
-      renderer->interactive_ops->get_text_width(renderer,
+      DIA_RENDERER_GET_CLASS(renderer)->get_text_width(renderer,
 						text->line[text->cursor_row],
 						text->strlen[text->cursor_row]);
-
     curs_x = text->position.x + str_width_first;
 
     switch (text->alignment) {
@@ -435,11 +466,11 @@ text_draw(Text *text, Renderer *renderer)
     p1.x = curs_x;
     p1.y = curs_y;
     p2.x = curs_x;
-    p2.y = curs_y + text->height;
+    p2.y = curs_y + height;
     
-    renderer->ops->set_linestyle(renderer, LINESTYLE_SOLID);
-    renderer->ops->set_linewidth(renderer, 0.1);
-    renderer->ops->draw_line(renderer, &p1, &p2, &color_black);
+    DIA_RENDERER_GET_CLASS(renderer)->set_linestyle(renderer, LINESTYLE_SOLID);
+    DIA_RENDERER_GET_CLASS(renderer)->set_linewidth(renderer, height/CURSOR_HEIGHT_RATIO);
+    DIA_RENDERER_GET_CLASS(renderer)->draw_line(renderer, &p1, &p2, &color_black);
   }
 }
 
@@ -459,7 +490,7 @@ text_set_cursor_at_end( Text* text )
 
 void
 text_set_cursor(Text *text, Point *clicked_point,
-		Renderer *renderer)
+		DiaRenderer *renderer)
 {
   real str_width_whole;
   real str_width_first;
@@ -487,9 +518,9 @@ text_set_cursor(Text *text, Point *clicked_point,
     return;
   }
 
-  renderer->ops->set_font(renderer, text->font, text->height);
+  DIA_RENDERER_GET_CLASS(renderer)->set_font(renderer, text->font, text->height);
   str_width_whole =
-    renderer->interactive_ops->get_text_width(renderer,
+    DIA_RENDERER_GET_CLASS(renderer)->get_text_width(renderer,
 					      text->line[row],
 					      text->strlen[row]);
   start_x = text->position.x;
@@ -509,7 +540,7 @@ text_set_cursor(Text *text, Point *clicked_point,
   
   for (i=0;i<=text->strlen[row];i++) {
     str_width_first =
-      renderer->interactive_ops->get_text_width(renderer, text->line[row], i);
+      DIA_RENDERER_GET_CLASS(renderer)->get_text_width(renderer, text->line[row], i);
     if (clicked_point->x - start_x >= str_width_first) {
       text->cursor_pos = i;
     } else {
@@ -523,13 +554,17 @@ static void
 text_join_lines(Text *text, int first_line)
 {
   int i;
-  int len1;
+  int len1, len2, alloclen1, alloclen2;
   real width;
   char *str1, *str2;
   int numlines;
 
   str1 = text->line[first_line];
   str2 = text->line[first_line+1];
+  len1 = text->strlen[first_line];
+  len2 = text->strlen[first_line + 1];
+  alloclen1 = text->alloclen[first_line];
+  alloclen2 = text->alloclen[first_line + 1];
 
   text->line[first_line] = NULL;
   text->line[first_line+1] = NULL;
@@ -540,9 +575,8 @@ text_join_lines(Text *text, int first_line)
     text->alloclen[i] = text->alloclen[i+1];
     text->row_width[i] = text->row_width[i+1];
   }
-  len1 = strlen(str1);
-  text->strlen[first_line] = len1 + strlen(str2);
-  text->alloclen[first_line] = text->strlen[first_line] + 1;
+  text->strlen[first_line] = len1 + len2;
+  text->alloclen[first_line] = alloclen1 + alloclen2;
   text->line[first_line] =
     g_malloc(sizeof(char)*(text->alloclen[first_line]));
   strcpy(text->line[first_line], str1);
@@ -558,7 +592,7 @@ text_join_lines(Text *text, int first_line)
   text->row_width = g_realloc(text->row_width, sizeof(real)*numlines);
 
   text->row_width[first_line] = 
-    font_string_width(text->line[first_line], text->font, text->height);
+    dia_font_string_width(text->line[first_line], text->font, text->height);
 
   width = 0.0;
   for (i=0;i<text->numlines;i++) {
@@ -577,6 +611,8 @@ text_delete_forward(Text *text)
   int row;
   int i;
   real width;
+  gchar *start, *end;
+  int len;
   
   row = text->cursor_row;
   
@@ -585,16 +621,23 @@ text_delete_forward(Text *text)
       text_join_lines(text, row);
     return;
   }
-  memmove(text->line[row] + text->cursor_pos,
-	  text->line[row] + text->cursor_pos + 1,
-	  text->strlen[row] - text->cursor_pos);
+  start = text->line[row];
+  for (i = 0; i < text->cursor_pos; i++) {
+    start = g_utf8_next_char (start);
+  }
+  end = g_utf8_next_char (start);
 
-  text->strlen[row]--;
+  len = strlen (text->line[row]);
+  memmove (start, end, text->line[row] + len - start);
+
+  text->strlen[row] = g_utf8_strlen(text->line[row], -1);//text->strlen[row]--;
   
   if (text->cursor_pos > text->strlen[text->cursor_row])
     text->cursor_pos = text->strlen[text->cursor_row];
 
-  text->row_width[row] = font_string_width(text->line[row], text->font, text->height);
+  text->row_width[row] = dia_font_string_width(text->line[row],
+                                              text->font,
+                                              text->height);
   width = 0.0;
   for (i=0;i<text->numlines;i++) {
     width = MAX(width, text->row_width[i]);
@@ -608,6 +651,8 @@ text_delete_backward(Text *text)
   int row;
   int i;
   real width;
+  gchar *start, *end;
+  int len;
   
   row = text->cursor_row;
   
@@ -616,18 +661,22 @@ text_delete_backward(Text *text)
       text_join_lines(text, row-1);
     return;
   }
-  memmove(text->line[row] + text->cursor_pos - 1,
-	  text->line[row] + text->cursor_pos,
-	  text->strlen[row] - text->cursor_pos + 1);
+  start = g_utf8_offset_to_pointer(text->line[row], 
+				   (glong)(text->cursor_pos-1));
+  end = g_utf8_offset_to_pointer(start, 1);
+  len = g_utf8_offset_to_pointer(text->line[row], text->strlen[row]) - end;
+  memmove (start, end, len + 1);
 
-  text->strlen[row]--;
+  text->strlen[row] = g_utf8_strlen(text->line[row], -1);
 
   text->cursor_pos --;
   
   if (text->cursor_pos > text->strlen[text->cursor_row])
     text->cursor_pos = text->strlen[text->cursor_row];
 
-  text->row_width[row] = font_string_width(text->line[row], text->font, text->height);
+  text->row_width[row] = dia_font_string_width(text->line[row],
+                                              text->font,
+                                              text->height);
 
   width = 0.0;
   for (i=0;i<text->numlines;i++) {
@@ -645,6 +694,8 @@ text_split_line(Text *text)
   char *line;
   int orig_len;
   real width;
+  gchar *str;
+  int orig_alloclen;
   
   text->numlines += 1;
   numlines = text->numlines;
@@ -663,25 +714,26 @@ text_split_line(Text *text)
   /* row and row+1 needs to be changed: */
   line = text->line[row];
   orig_len = text->strlen[row];
+  orig_alloclen = text->alloclen[row];
   
   text->strlen[row] = text->cursor_pos;
-  text->alloclen[row] = text->strlen[row] + 1;
-  text->line[row] = g_malloc(sizeof(char)*(text->alloclen[row]));
-  memcpy(text->line[row], line, text->strlen[row]);
-  text->line[row][text->strlen[row]] = 0;
+  str = text->line[row];
+  for (i = 0; i < text->cursor_pos; i++) {
+    str = g_utf8_next_char (str);
+  }
+  text->alloclen[row] = str - text->line[row] + 1;
+  text->line[row] = g_strndup (line, str - text->line[row]);
   
-  text->strlen[row+1] = orig_len - text->strlen[row];
-  text->alloclen[row+1] = text->strlen[row+1] + 1;
-  text->line[row+1] = g_malloc(sizeof(char)*(text->alloclen[row+1]));
-  memcpy(text->line[row+1], line + text->cursor_pos, text->strlen[row+1]);
-  text->line[row+1][text->strlen[row+1]] = 0;
+  text->strlen[row + 1] = orig_len - text->strlen[row];
+  text->alloclen[row + 1] = orig_alloclen - strlen (text->line[row]) + 1;
+  text->line[row + 1] = g_strndup (str, text->alloclen[row + 1] - 1);
 
   g_free(line);
 
   text->row_width[row] = 
-    font_string_width(text->line[row], text->font, text->height);
+    dia_font_string_width(text->line[row], text->font, text->height);
   text->row_width[row+1] = 
-    font_string_width(text->line[row+1], text->font, text->height);
+    dia_font_string_width(text->line[row+1], text->font, text->height);
 
   width = 0.0;
   for (i=0;i<text->numlines;i++) {
@@ -694,138 +746,158 @@ text_split_line(Text *text)
 }
 
 static void
-text_insert_char(Text *text, char c)
+text_insert_char(Text *text, gunichar c)
 {
   int row;
   int i;
-  char *line;
+  gchar *line, *str;
+  gchar ch[7];
+  int unilen, length;
+
+  unilen = g_unichar_to_utf8 (c, ch);
+  ch[unilen] = 0;
   
   row = text->cursor_row;
 
-  if ( text->strlen[row]+2 > text->alloclen[row] ) {
-    text->alloclen[row] = text->strlen[row]*2+2;
-    text->line[row] =
-      g_realloc( text->line[row], sizeof(char)*text->alloclen[row] );
+  length = strlen (text->line[row]);
+  if ((length + unilen + 1) > text->alloclen[row]) {
+    text->alloclen[row] = length * 2 + unilen + 1;
+    text->line[row] = g_realloc (text->line[row], sizeof (gchar) * text->alloclen[row]);
+  }
+
+  str = text->line[row];
+  for (i = 0; i < text->cursor_pos; i++) {
+    str = g_utf8_next_char (str);
   }
 
   line = text->line[row];
-  for (i=text->strlen[row];i>=text->cursor_pos;i--) {
-    line[i+1] = line[i];
+  /* copy the part to the right of the insertion */
+  for (i = length; &line[i] >= str; i--) {
+    line[i + unilen] = line[i];
   }
-  line[text->cursor_pos] = c;
+  strncpy (str, ch, unilen);
+  line[length + unilen] = 0; /* null terminate */
   text->cursor_pos += 1;
-  text->strlen[row] += 1;
+  text->strlen[row] = g_utf8_strlen(text->line[row], -1);// length + unilen;
 
-  text->row_width[row] = font_string_width(text->line[row], text->font, text->height);
+  text->row_width[row] =
+      dia_font_string_width(text->line[row], text->font, text->height);
   text->max_width = MAX(text->max_width, text->row_width[row]);
 }
 
 static int
-text_key_event(Focus *focus, guint keyval, char *str, int strlen,
-	       ObjectChange **change)
+text_key_event(Focus *focus, guint keyval, const gchar *str, int strlen,
+               ObjectChange **change)
 {
   Text *text;
-  int return_val;
-  int row;
+  int return_val = FALSE;
+  int row, i;
+  const char *utf;
+  gunichar c;
 
-  return_val = FALSE;
   *change = NULL;
   
   text = (Text *)focus->user_data;
 
-  /*
-  printf("Got an %d '%s' (%d)\n", keyval, str, strlen);
-  */
-  
   switch(keyval) {
-  case GDK_Up:
-    text->cursor_row--;
-    if (text->cursor_row<0)
-      text->cursor_row = 0;
+      case GDK_Up:
+        text->cursor_row--;
+        if (text->cursor_row<0)
+          text->cursor_row = 0;
 
-    if (text->cursor_pos > text->strlen[text->cursor_row])
-      text->cursor_pos = text->strlen[text->cursor_row];
+        if (text->cursor_pos > text->strlen[text->cursor_row])
+          text->cursor_pos = text->strlen[text->cursor_row];
 
-    break;
-  case GDK_Down:
-    text->cursor_row++;
-    if (text->cursor_row >= text->numlines)
-      text->cursor_row = text->numlines - 1;
+        break;
+      case GDK_Down:
+        text->cursor_row++;
+        if (text->cursor_row >= text->numlines)
+          text->cursor_row = text->numlines - 1;
 
-    if (text->cursor_pos > text->strlen[text->cursor_row])
-      text->cursor_pos = text->strlen[text->cursor_row];
+        if (text->cursor_pos > text->strlen[text->cursor_row])
+          text->cursor_pos = text->strlen[text->cursor_row];
     
-    break;
-  case GDK_Left:
-    text->cursor_pos--;
-    if (text->cursor_pos<0)
-      text->cursor_pos = 0;
-    break;
-  case GDK_Right:
-    text->cursor_pos++;
-    if (text->cursor_pos > text->strlen[text->cursor_row])
-      text->cursor_pos = text->strlen[text->cursor_row];
-    break;
-  case GDK_Home:
-    text->cursor_pos = 0;
-    break;
-  case GDK_End:
-    text->cursor_pos = text->strlen[text->cursor_row];
-    break;
-  case GDK_Delete:
-    return_val = TRUE;
-    row = text->cursor_row;
-    if (text->cursor_pos >= text->strlen[row]) {
-      if (row+1 < text->numlines) {
-	*change = text_create_change(text, TYPE_JOIN_ROW, 'Q',
-				     text->cursor_pos, row);
-      } else {
-	return_val = FALSE;
-	break;
-      }
-    } else {
-      *change = text_create_change(text, TYPE_DELETE_FORWARD,
-				   text->line[row][text->cursor_pos],
-				   text->cursor_pos, text->cursor_row);
-    }
-    text_delete_forward(text);
-    break;
-  case GDK_BackSpace:
-    return_val = TRUE;
-    row = text->cursor_row;
-    if (text->cursor_pos <= 0) {
-      if (row > 0) {
-	*change = text_create_change(text, TYPE_JOIN_ROW, 'Q',
-				     text->strlen[row-1], row-1);
-      } else {
-	return_val = FALSE;
-	break;
-      }
-    } else {
-      *change = text_create_change(text, TYPE_DELETE_BACKWARD,
-				   text->line[row][text->cursor_pos-1],
-				   text->cursor_pos-1, text->cursor_row);
-    }
-    text_delete_backward(text);
-    break;
-  case GDK_Return:
-    return_val = TRUE;
-    *change = text_create_change(text, TYPE_SPLIT_ROW, 'Q',
-				 text->cursor_pos, text->cursor_row);
-    text_split_line(text);
-    break;
-  default:
-    if (strlen>0) {
-      int i;
+        break;
+      case GDK_Left:
+        text->cursor_pos--;
+        if (text->cursor_pos<0)
+          text->cursor_pos = 0;
+        break;
+      case GDK_Right:
+        text->cursor_pos++;
+        if (text->cursor_pos > text->strlen[text->cursor_row])
+          text->cursor_pos = text->strlen[text->cursor_row];
+        break;
+      case GDK_Home:
+        text->cursor_pos = 0;
+        break;
+      case GDK_End:
+        text->cursor_pos = text->strlen[text->cursor_row];
+        break;
+      case GDK_Delete:
+        return_val = TRUE;
+        row = text->cursor_row;
+        if (text->cursor_pos >= text->strlen[row]) {
+          if (row+1 < text->numlines) {
+            *change = text_create_change(text, TYPE_JOIN_ROW, 'Q',
+                                         text->cursor_pos, row);
+          } else {
+            return_val = FALSE;
+            break;
+          }
+        } else {
+          utf = text->line[row];
+          for (i = 0; i < text->cursor_pos; i++)
+            utf = g_utf8_next_char (utf);
+          c = g_utf8_get_char (utf);
+          *change = text_create_change (text, TYPE_DELETE_FORWARD, c,
+                                        text->cursor_pos, text->cursor_row);
+        }
+        text_delete_forward(text);
+        break;
+      case GDK_BackSpace:
+        return_val = TRUE;
+        row = text->cursor_row;
+        if (text->cursor_pos <= 0) {
+          if (row > 0) {
+            *change = text_create_change(text, TYPE_JOIN_ROW, 'Q',
+                                         text->strlen[row-1], row-1);
+          } else {
+            return_val = FALSE;
+            break;
+          }
+        } else {
+          utf = text->line[row];
+          for (i = 0; i < (text->cursor_pos - 1); i++)
+            utf = g_utf8_next_char (utf);
+          c = g_utf8_get_char (utf);
+          *change = text_create_change (text, TYPE_DELETE_BACKWARD, c,
+                                        text->cursor_pos - 1,
+                                        text->cursor_row);
+        }
+        text_delete_backward(text);
+        break;
+      case GDK_Return:
+        return_val = TRUE;
+        *change = text_create_change(text, TYPE_SPLIT_ROW, 'Q',
+                                     text->cursor_pos, text->cursor_row);
+        text_split_line(text);
+        break;
+      default:
+        if (str || (strlen>0)) {
 
-      return_val = TRUE;
-      for (i = 0; i < strlen; i++) {
-	*change = text_create_change(text, TYPE_INSERT_CHAR, str[i],
-				     text->cursor_pos, text->cursor_row);
-	text_insert_char(text, str[i]);
-      }
-    }
-    break;
+          return_val = TRUE;
+          utf = str;
+          for (utf = str ; utf && *utf && (utf-str <= strlen) ; 
+	       utf = g_utf8_next_char (utf)) {
+            c = g_utf8_get_char (utf);
+            
+            *change = text_create_change (text, TYPE_INSERT_CHAR, c,
+                                          text->cursor_pos, text->cursor_row);
+            text_insert_char (text, c);
+          }
+        }
+        break;
   }  
   
   return return_val;
@@ -850,6 +922,7 @@ text_delete_all(Text *text, ObjectChange **change)
 				 0, text->cursor_pos, text->cursor_row);
     
     text_set_string(text, "");
+    calc_ascent_descent(text);
     return TRUE;
   }
   return FALSE;
@@ -884,7 +957,7 @@ Text *
 data_text(AttributeNode text_attr)
 {
   char *string = "";
-  Font *font;
+  DiaFont *font;
   real height;
   Point pos = {0.0, 0.0};
   Color col;
@@ -899,16 +972,18 @@ data_text(AttributeNode text_attr)
   if (attr != NULL)
     string = data_string(attribute_first_data(attr));
 
-  font = font_getfont("Courier");
-  attr = composite_find_attribute(text_attr, "font");
-  if (attr != NULL)
-    font = data_font(attribute_first_data(attr));
-
   height = 1.0;
   attr = composite_find_attribute(text_attr, "height");
   if (attr != NULL)
     height = data_real(attribute_first_data(attr));
 
+  attr = composite_find_attribute(text_attr, "font");
+  if (attr != NULL) {
+    font = data_font(attribute_first_data(attr));
+  } else {
+    font = dia_font_new_from_style(DIA_FONT_SANS,1.0);
+  }
+  
   attr = composite_find_attribute(text_attr, "pos");
   if (attr != NULL)
     data_point(attribute_first_data(attr), &pos);
@@ -924,13 +999,14 @@ data_text(AttributeNode text_attr)
     align = data_enum(attribute_first_data(attr));
   
   text = new_text(string, font, height, &pos, &col, align);
+  if (font) dia_font_unref(font);
   if (string) g_free(string);
   return text;
 }
 
 void
 text_get_attributes(Text *text, TextAttributes *attr)
-{
+{    
   attr->font = text->font;
   attr->height = text->height;
   attr->position = text->position;
@@ -941,7 +1017,10 @@ text_get_attributes(Text *text, TextAttributes *attr)
 void
 text_set_attributes(Text *text, TextAttributes *attr)
 {
-  text->font = attr->font;
+  if (text->font != attr->font) {
+      dia_font_unref(text->font);
+      text->font = dia_font_ref(attr->font);
+  }
   text->height = attr->height;
   text->position = attr->position;
   text->color = attr->color;
@@ -1029,7 +1108,7 @@ text_change_free(struct TextObjectChange *change) {
 
 static ObjectChange *
 text_create_change(Text *text, enum change_type type,
-		   char ch, int pos, int row)
+		   gunichar ch, int pos, int row)
 {
   struct TextObjectChange *change;
 
@@ -1050,3 +1129,38 @@ text_create_change(Text *text, enum change_type type,
     change->str = NULL;
   return (ObjectChange *)change;
 }
+
+gboolean 
+apply_textattr_properties(GPtrArray *props,
+                          Text *text, const gchar *textname,
+                          TextAttributes *attrs)
+{
+  TextProperty *textprop = 
+    (TextProperty *)find_prop_by_name_and_type(props,textname,PROP_TYPE_TEXT);
+
+  if ((!textprop) || 
+      ((textprop->common.experience & (PXP_LOADED|PXP_SFO))==0 )) {
+    /* most likely we're called after the dialog box has been applied */
+    text_set_attributes(text,attrs);
+    return TRUE; 
+  }
+  return FALSE;
+}
+
+gboolean 
+apply_textstr_properties(GPtrArray *props,
+                         Text *text, const gchar *textname,
+                         const gchar *str)
+{
+  TextProperty *textprop = 
+    (TextProperty *)find_prop_by_name_and_type(props,textname,PROP_TYPE_TEXT);
+
+  if ((!textprop) || 
+      ((textprop->common.experience & (PXP_LOADED|PXP_SFO))==0 )) {
+    /* most likely we're called after the dialog box has been applied */
+    text_set_string(text,str);
+    return TRUE; 
+  }
+  return FALSE;
+}
+
