@@ -16,12 +16,13 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
+/* DO NOT USE THIS OBJECT AS A BASIS FOR A NEW OBJECT. */
+
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
 
 #include <assert.h>
-#include <gtk/gtk.h>
 #include <math.h>
 #include <string.h>
 
@@ -29,7 +30,7 @@
 #include "object.h"
 #include "element.h"
 #include "connectionpoint.h"
-#include "render.h"
+#include "diarenderer.h"
 #include "attributes.h"
 #include "widgets.h"
 #include "properties.h"
@@ -49,7 +50,8 @@ typedef struct _Relationship Relationship;
 struct _Relationship {
   Element element;
 
-  Font *font;
+  DiaFont *font;
+  real font_height;
   gchar *name;
   gchar *left_cardinality;
   gchar *right_cardinality;
@@ -71,11 +73,12 @@ struct _Relationship {
 
 static real relationship_distance_from(Relationship *relationship, Point *point);
 static void relationship_select(Relationship *relationship, Point *clicked_point,
-		       Renderer *interactive_renderer);
-static void relationship_move_handle(Relationship *relationship, Handle *handle,
-			    Point *to, HandleMoveReason reason, ModifierKeys modifiers);
-static void relationship_move(Relationship *relationship, Point *to);
-static void relationship_draw(Relationship *relationship, Renderer *renderer);
+		       DiaRenderer *interactive_renderer);
+static ObjectChange* relationship_move_handle(Relationship *relationship, Handle *handle,
+					      Point *to, ConnectionPoint *cp,
+					      HandleMoveReason reason, ModifierKeys modifiers);
+static ObjectChange* relationship_move(Relationship *relationship, Point *to);
+static void relationship_draw(Relationship *relationship, DiaRenderer *renderer);
 static void relationship_update_data(Relationship *relationship);
 static Object *relationship_create(Point *startpoint,
 			  void *user_data,
@@ -91,9 +94,9 @@ static Object *relationship_load(ObjectNode obj_node, int version,
 static PropDescription *
 relationship_describe_props(Relationship *relationship);
 static void
-relationship_get_props(Relationship *relationship, Property *props, guint nprops);
+relationship_get_props(Relationship *relationship, GPtrArray *props);
 static void
-relationship_set_props(Relationship *relationship, Property *props, guint nprops);
+relationship_set_props(Relationship *relationship, GPtrArray *props);
 
 static ObjectTypeOps relationship_type_ops =
 {
@@ -144,6 +147,8 @@ static PropDescription relationship_props[] = {
   PROP_STD_LINE_WIDTH,
   PROP_STD_LINE_COLOUR,
   PROP_STD_FILL_COLOUR,
+  PROP_STD_TEXT_FONT,
+  PROP_STD_TEXT_HEIGHT,
   PROP_DESC_END
 };
 
@@ -165,22 +170,24 @@ static PropOffset relationship_offsets[] = {
   { "line_width", PROP_TYPE_REAL, offsetof(Relationship, border_width) },
   { "line_colour", PROP_TYPE_COLOUR, offsetof(Relationship, border_color) },
   { "fill_colour", PROP_TYPE_COLOUR, offsetof(Relationship, inner_color) },
+  { "text_font", PROP_TYPE_FONT, offsetof (Relationship, font) },
+  { "text_height", PROP_TYPE_REAL, offsetof(Relationship, font_height) },
   { NULL, 0, 0}
 };
 
 
 static void
-relationship_get_props(Relationship *relationship, Property *props, guint nprops)
+relationship_get_props(Relationship *relationship, GPtrArray *props)
 {
   object_get_props_from_offsets(&relationship->element.object, 
-                                relationship_offsets, props, nprops);
+                                relationship_offsets, props);
 }
 
 static void
-relationship_set_props(Relationship *relationship, Property *props, guint nprops)
+relationship_set_props(Relationship *relationship, GPtrArray *props)
 {
   object_set_props_from_offsets(&relationship->element.object, 
-                                relationship_offsets, props, nprops);
+                                relationship_offsets, props);
   relationship_update_data(relationship);
 }
 
@@ -200,35 +207,41 @@ relationship_distance_from(Relationship *relationship, Point *point)
 
 static void
 relationship_select(Relationship *relationship, Point *clicked_point,
-	   Renderer *interactive_renderer)
+	   DiaRenderer *interactive_renderer)
 {
   element_update_handles(&relationship->element);
 }
 
-static void
+static ObjectChange*
 relationship_move_handle(Relationship *relationship, Handle *handle,
-		Point *to, HandleMoveReason reason, ModifierKeys modifiers)
+			 Point *to, ConnectionPoint *cp,
+			 HandleMoveReason reason, ModifierKeys modifiers)
 {
   assert(relationship!=NULL);
   assert(handle!=NULL);
   assert(to!=NULL);
 
-  element_move_handle(&relationship->element, handle->id, to, reason);
+  element_move_handle(&relationship->element, handle->id, to, cp, reason, modifiers);
 
   relationship_update_data(relationship);
+
+  return NULL;
 }
 
-static void
+static ObjectChange*
 relationship_move(Relationship *relationship, Point *to)
 {
   relationship->element.corner = *to;
   
   relationship_update_data(relationship);
+
+  return NULL;
 }
 
 static void
-relationship_draw(Relationship *relationship, Renderer *renderer)
+relationship_draw(Relationship *relationship, DiaRenderer *renderer)
 {
+  DiaRendererClass *renderer_ops = DIA_RENDERER_GET_CLASS (renderer);
   Point corners[4];
   Point lc, rc;
   Point p;
@@ -250,23 +263,23 @@ relationship_draw(Relationship *relationship, Renderer *renderer)
   corners[3].x = elem->corner.x + elem->width / 2;
   corners[3].y = elem->corner.y + elem->height;
 
-  renderer->ops->set_fillstyle(renderer, FILLSTYLE_SOLID);
+  renderer_ops->set_fillstyle(renderer, FILLSTYLE_SOLID);
 
-  renderer->ops->fill_polygon(renderer, corners, 4, 
+  renderer_ops->fill_polygon(renderer, corners, 4, 
 			      &relationship->inner_color);
 
-  renderer->ops->set_linewidth(renderer, relationship->border_width);
-  renderer->ops->set_linestyle(renderer, LINESTYLE_SOLID);
-  renderer->ops->set_linejoin(renderer, LINEJOIN_MITER);
+  renderer_ops->set_linewidth(renderer, relationship->border_width);
+  renderer_ops->set_linestyle(renderer, LINESTYLE_SOLID);
+  renderer_ops->set_linejoin(renderer, LINEJOIN_MITER);
 
-  renderer->ops->draw_polygon(renderer, corners, 4, 
+  renderer_ops->draw_polygon(renderer, corners, 4, 
 			      &relationship->border_color);
   
   if (relationship->rotate) {
     lc.x = corners[1].x + 0.2;
     lc.y = corners[1].y - 0.3;
     rc.x = corners[3].x + 0.2;
-    rc.y = corners[3].y + 0.3 + FONT_HEIGHT;
+    rc.y = corners[3].y + 0.3 + relationship->font_height;
     left_align = ALIGN_LEFT;
   } else {
     lc.x = corners[0].x - CARDINALITY_DISTANCE;
@@ -283,25 +296,26 @@ relationship_draw(Relationship *relationship, Renderer *renderer)
     corners[2].x -= diff;
     corners[3].y -= diff*DIAMOND_RATIO;
 
-    renderer->ops->draw_polygon(renderer, corners, 4,
+    renderer_ops->draw_polygon(renderer, corners, 4,
 				&relationship->border_color);
   }
 
-  renderer->ops->set_font(renderer, relationship->font, FONT_HEIGHT);
-  renderer->ops->draw_string(renderer,
+  renderer_ops->set_font(renderer, relationship->font, relationship->font_height);
+  renderer_ops->draw_string(renderer,
 			     relationship->left_cardinality,
 			     &lc, left_align,
 			     &color_black);
-  renderer->ops->draw_string(renderer,
+  renderer_ops->draw_string(renderer,
 			     relationship->right_cardinality,
 			     &rc, ALIGN_LEFT,
 			     &color_black);
 
   p.x = elem->corner.x + elem->width / 2.0;
-  p.y = elem->corner.y + (elem->height - FONT_HEIGHT)/2.0 +
-         font_ascent(relationship->font, FONT_HEIGHT);
+  p.y = elem->corner.y + (elem->height - relationship->font_height)/2.0 +
+         dia_font_ascent(relationship->name,
+                         relationship->font, relationship->font_height);
   
-  renderer->ops->draw_string(renderer, 
+  renderer_ops->draw_string(renderer, 
 			     relationship->name, 
 			     &p, ALIGN_CENTER, 
 			     &color_black);
@@ -315,11 +329,11 @@ relationship_update_data(Relationship *relationship)
   ElementBBExtras *extra = &elem->extra_spacing;
 
   relationship->name_width =
-    font_string_width(relationship->name, relationship->font, FONT_HEIGHT);
+    dia_font_string_width(relationship->name, relationship->font, relationship->font_height);
   relationship->left_card_width =
-    font_string_width(relationship->left_cardinality, relationship->font, FONT_HEIGHT);
+    dia_font_string_width(relationship->left_cardinality, relationship->font, relationship->font_height);
   relationship->right_card_width =
-    font_string_width(relationship->right_cardinality, relationship->font, FONT_HEIGHT);
+    dia_font_string_width(relationship->right_cardinality, relationship->font, relationship->font_height);
 
   elem->width = relationship->name_width + 2*TEXT_BORDER_WIDTH_X;
   elem->height = elem->width * DIAMOND_RATIO;
@@ -339,37 +353,53 @@ relationship_update_data(Relationship *relationship)
         6    
    */
   
-  relationship->connections[0].pos.x = elem->corner.x;
-  relationship->connections[0].pos.y = elem->corner.y + elem->height / 2.0;
+  connpoint_update(&relationship->connections[0],
+		   elem->corner.x,
+		   elem->corner.y + elem->height / 2.0,
+		   DIR_WEST|DIR_NORTH|DIR_SOUTH);
 
-  relationship->connections[1].pos.x = elem->corner.x + elem->width / 4.0;
-  relationship->connections[1].pos.y = elem->corner.y + elem->height / 4.0;
+  connpoint_update(&relationship->connections[1],
+		   elem->corner.x + elem->width / 4.0,
+		   elem->corner.y + elem->height / 4.0,
+		   DIR_NORTHWEST);
 
-  relationship->connections[2].pos.x = elem->corner.x + elem->width / 2.0;
-  relationship->connections[2].pos.y = elem->corner.y;
+  connpoint_update(&relationship->connections[2],
+		   elem->corner.x + elem->width / 2.0,
+		   elem->corner.y,
+		   DIR_NORTH|DIR_EAST|DIR_WEST);
   
-  relationship->connections[3].pos.x = elem->corner.x + 3.0 * elem->width / 4.0;
-  relationship->connections[3].pos.y = elem->corner.y + elem->height / 4.0;
+  connpoint_update(&relationship->connections[3],
+		   elem->corner.x + 3.0 * elem->width / 4.0,
+		   elem->corner.y + elem->height / 4.0,
+		   DIR_NORTHEAST);
   
-  relationship->connections[4].pos.x = elem->corner.x + elem->width;
-  relationship->connections[4].pos.y = elem->corner.y + elem->height / 2.0;
+  connpoint_update(&relationship->connections[4],
+		   elem->corner.x + elem->width,
+		   elem->corner.y + elem->height / 2.0,
+		   DIR_EAST|DIR_NORTH|DIR_SOUTH);
 
-  relationship->connections[5].pos.x = elem->corner.x + 3.0 * elem->width / 4.0;
-  relationship->connections[5].pos.y = elem->corner.y + 3.0 * elem->height / 4.0;
+  connpoint_update(&relationship->connections[5],
+		   elem->corner.x + 3.0 * elem->width / 4.0,
+		   elem->corner.y + 3.0 * elem->height / 4.0,
+		   DIR_SOUTHEAST);
   
-  relationship->connections[6].pos.x = elem->corner.x + elem->width / 2.0;
-  relationship->connections[6].pos.y = elem->corner.y + elem->height;
+  connpoint_update(&relationship->connections[6],
+		   elem->corner.x + elem->width / 2.0,
+		   elem->corner.y + elem->height,
+		   DIR_SOUTH|DIR_EAST|DIR_WEST);
 
-  relationship->connections[7].pos.x = elem->corner.x + elem->width / 4.0;
-  relationship->connections[7].pos.y = elem->corner.y + 3.0 * elem->height / 4.0;
+  connpoint_update(&relationship->connections[7],
+		   elem->corner.x + elem->width / 4.0,
+		   elem->corner.y + 3.0 * elem->height / 4.0,
+		   DIR_SOUTHWEST);
 
   extra->border_trans = relationship->border_width / 2.0;
   element_update_boundingbox(elem);
   
   /* fix boundingrelationship for line_width: */
   if(relationship->rotate) {
-    obj->bounding_box.top -= FONT_HEIGHT + CARDINALITY_DISTANCE;
-    obj->bounding_box.bottom += FONT_HEIGHT + CARDINALITY_DISTANCE;
+    obj->bounding_box.top -= relationship->font_height + CARDINALITY_DISTANCE;
+    obj->bounding_box.bottom += relationship->font_height + CARDINALITY_DISTANCE;
   }
   else {
     obj->bounding_box.left -= CARDINALITY_DISTANCE + relationship->left_card_width;
@@ -400,7 +430,7 @@ relationship_create(Point *startpoint,
 
   elem->corner = *startpoint;
   elem->width = DEFAULT_WIDTH;
-  elem->height = DEFAULT_WIDTH;
+  elem->height = DEFAULT_HEIGHT;
 
   relationship->border_width =  attributes_get_default_linewidth();
   relationship->border_color = attributes_get_foreground();
@@ -414,20 +444,14 @@ relationship_create(Point *startpoint,
     relationship->connections[i].connected = NULL;
   }
 
-  relationship->font = font_getfont("Courier");
+  relationship->font = dia_font_new_from_style(DIA_FONT_MONOSPACE,FONT_HEIGHT);
+  relationship->font_height = FONT_HEIGHT;
   relationship->name = g_strdup(_("Relationship"));
   relationship->left_cardinality = g_strdup("");
   relationship->right_cardinality = g_strdup("");
   relationship->identifying = FALSE;
   relationship->rotate = FALSE;
 
-  relationship->name_width =
-    font_string_width(relationship->name, relationship->font, FONT_HEIGHT);
-  relationship->left_card_width =
-    font_string_width(relationship->left_cardinality, relationship->font, FONT_HEIGHT);
-  relationship->right_card_width =
-    font_string_width(relationship->right_cardinality, relationship->font, FONT_HEIGHT);
-
   relationship_update_data(relationship);
 
   for (i=0;i<8;i++) {
@@ -442,6 +466,7 @@ relationship_create(Point *startpoint,
 static void
 relationship_destroy(Relationship *relationship)
 {
+  dia_font_unref(relationship->font);  
   element_destroy(&relationship->element);
   g_free(relationship->name);
   g_free(relationship->left_cardinality);
@@ -476,12 +501,13 @@ relationship_copy(Relationship *relationship)
     newrelationship->connections[i].last_pos = relationship->connections[i].last_pos;
   }
 
-  newrelationship->font = relationship->font;
-  newrelationship->name = strdup(relationship->name);
+  newrelationship->font = dia_font_ref(relationship->font);
+  newrelationship->font_height = relationship->font_height;
+  newrelationship->name = g_strdup(relationship->name);
   newrelationship->left_cardinality =
-    strdup(relationship->left_cardinality);
+    g_strdup(relationship->left_cardinality);
   newrelationship->right_cardinality =
-    strdup(relationship->right_cardinality);
+    g_strdup(relationship->right_cardinality);
   newrelationship->name_width = relationship->name_width;
   newrelationship->left_card_width = relationship->left_card_width;
   newrelationship->right_card_width = relationship->right_card_width;
@@ -514,6 +540,10 @@ relationship_save(Relationship *relationship, ObjectNode obj_node,
 		   relationship->identifying);
   data_add_boolean(new_attribute(obj_node, "rotated"),
 		   relationship->rotate);
+  data_add_font (new_attribute (obj_node, "font"),
+		 relationship->font);
+  data_add_real(new_attribute(obj_node, "font_height"),
+  		relationship->font_height);
 }
 
 static Object *
@@ -571,7 +601,17 @@ relationship_load(ObjectNode obj_node, int version, const char *filename)
   attr = object_find_attribute(obj_node, "rotated");
   if (attr != NULL)
     relationship->rotate = data_boolean(attribute_first_data(attr));
-  
+
+  relationship->font = NULL;
+  attr = object_find_attribute (obj_node, "font");
+  if (attr != NULL)
+	  relationship->font = data_font (attribute_first_data (attr));
+
+  relationship->font_height = FONT_HEIGHT;
+  attr = object_find_attribute(obj_node, "font_height");
+  if (attr != NULL)
+    relationship->font_height = data_real(attribute_first_data(attr));
+
   element_init(elem, 8, 8);
 
   for (i=0;i<8;i++) {
@@ -580,14 +620,10 @@ relationship_load(ObjectNode obj_node, int version, const char *filename)
     relationship->connections[i].connected = NULL;
   }
 
-  relationship->font = font_getfont("Courier");
-
-  relationship->name_width =
-    font_string_width(relationship->name, relationship->font, FONT_HEIGHT);
-  relationship->left_card_width =
-    font_string_width(relationship->left_cardinality, relationship->font, FONT_HEIGHT);
-  relationship->right_card_width =
-    font_string_width(relationship->right_cardinality, relationship->font, FONT_HEIGHT);
+  if (relationship->font == NULL) {
+    relationship->font = dia_font_new_from_style(DIA_FONT_MONOSPACE,
+                                                 FONT_HEIGHT);
+  }
 
   relationship_update_data(relationship);
 
@@ -597,3 +633,4 @@ relationship_load(ObjectNode obj_node, int version, const char *filename)
 
   return &relationship->element.object;
 }
+
