@@ -24,14 +24,13 @@
 #endif
 
 #include <assert.h>
-#include <gtk/gtk.h>
 #include <math.h>
 #include <string.h>
 
 #include "intl.h"
 #include "object.h"
 #include "element.h"
-#include "render.h"
+#include "diarenderer.h"
 #include "attributes.h"
 #include "text.h"
 #include "properties.h"
@@ -53,34 +52,33 @@ static const double BRANCH_WIDTH = 2.0;
 static const double BRANCH_HEIGHT = 2.0;
 
 static real branch_distance_from(Branch *branch, Point *point);
-static void branch_select(Branch *branch, Point *clicked_point, Renderer *interactive_renderer);
-static void branch_move_handle(Branch *branch, Handle *handle,
-			       Point *to, HandleMoveReason reason, ModifierKeys modifiers);
-static void branch_move(Branch *branch, Point *to);
-static void branch_draw(Branch *branch, Renderer *renderer);
+static void branch_select(Branch *branch, Point *clicked_point, DiaRenderer *interactive_renderer);
+static ObjectChange* branch_move_handle(Branch *branch, Handle *handle,
+					Point *to, ConnectionPoint *cp,
+					HandleMoveReason reason, ModifierKeys modifiers);
+static ObjectChange* branch_move(Branch *branch, Point *to);
+static void branch_draw(Branch *branch, DiaRenderer *renderer);
 static Object *branch_create(Point *startpoint,
 			     void *user_data,
 			     Handle **handle1,
 			     Handle **handle2);
 static void branch_destroy(Branch *branch);
-static Object *branch_copy(Branch *branch);
-
-static void branch_save(Branch *branch, ObjectNode obj_node,
-			const char *filename);
 static Object *branch_load(ObjectNode obj_node, int version,
 			   const char *filename);
 
 static PropDescription *branch_describe_props(Branch *branch);
-static void branch_get_props(Branch *branch, Property *props, guint nprops);
-static void branch_set_props(Branch *branch, Property *props, guint nprops);
+static void branch_get_props(Branch *branch, GPtrArray *props);
+static void branch_set_props(Branch *branch, GPtrArray *props);
 
 static void branch_update_data(Branch *branch);
 
 static ObjectTypeOps branch_type_ops =
 {
   (CreateFunc) branch_create,
-  (LoadFunc)   branch_load,
-  (SaveFunc)   branch_save
+  (LoadFunc)   branch_load,/*using_properties*/     /* load */
+  (SaveFunc)   object_save_using_properties,      /* save */
+  (GetDefaultsFunc)   NULL, 
+  (ApplyDefaultsFunc) NULL
 };
 
 ObjectType branch_type =
@@ -98,7 +96,7 @@ static ObjectOps branch_ops =
   (DrawFunc)            branch_draw,
   (DistanceFunc)        branch_distance_from,
   (SelectFunc)          branch_select,
-  (CopyFunc)            branch_copy,
+  (CopyFunc)            object_copy_using_properties,
   (MoveFunc)            branch_move,
   (MoveHandleFunc)      branch_move_handle,
   (GetPropertiesFunc)   object_return_null,
@@ -118,8 +116,9 @@ static PropDescription branch_props[] = {
 static PropDescription *
 branch_describe_props(Branch *branch)
 {
-  if (branch_props[0].quark == 0)
+  if (branch_props[0].quark == 0) {
     prop_desc_list_calculate_quarks(branch_props);
+  }
   return branch_props;
 }
 
@@ -129,21 +128,17 @@ static PropOffset branch_offsets[] = {
 };
 
 static void
-branch_get_props(Branch * branch, Property *props, guint nprops)
+branch_get_props(Branch * branch, GPtrArray *props)
 {
-  if (object_get_props_from_offsets(&branch->element.object, 
-                                    branch_offsets, props, nprops))
-    return;
-  /* none yet */
+  object_get_props_from_offsets(&branch->element.object, 
+                                branch_offsets, props);
 }
 
 static void
-branch_set_props(Branch *branch, Property *props, guint nprops)
+branch_set_props(Branch *branch, GPtrArray *props)
 {
-  if (!object_set_props_from_offsets(&branch->element.object, 
-                                     branch_offsets, props, nprops)) {
-    /* none yet */
-  }
+  object_set_props_from_offsets(&branch->element.object, 
+                                branch_offsets, props);
   branch_update_data(branch);
 }
 
@@ -155,14 +150,15 @@ branch_distance_from(Branch *branch, Point *point)
 }
 
 static void
-branch_select(Branch *branch, Point *clicked_point, Renderer *interactive_renderer)
+branch_select(Branch *branch, Point *clicked_point, DiaRenderer *interactive_renderer)
 {
   element_update_handles(&branch->element);
 }
 
-static void
+static ObjectChange*
 branch_move_handle(Branch *branch, Handle *handle,
-			 Point *to, HandleMoveReason reason, ModifierKeys modifiers)
+		   Point *to, ConnectionPoint *cp,
+		   HandleMoveReason reason, ModifierKeys modifiers)
 {
   assert(branch!=NULL);
   assert(handle!=NULL);
@@ -170,19 +166,30 @@ branch_move_handle(Branch *branch, Handle *handle,
 
   assert(handle->id < 8);
   
-  element_move_handle(&branch->element, handle->id, to, reason);
+  /* It smashes size info in update_data anyway.  And none of its siblings
+   * resizable, so until that changes, this should be properly unresizable
+   */
+    
+  /*
+  element_move_handle(&branch->element, handle->id, to, cp, reason, modifiers);
   branch_update_data(branch);
+  */
+
+  return NULL;
 }
 
-static void
+static ObjectChange*
 branch_move(Branch *branch, Point *to)
 {
   branch->element.corner = *to;
   branch_update_data(branch);
+
+  return NULL;
 }
 
-static void branch_draw(Branch *branch, Renderer *renderer)
+static void branch_draw(Branch *branch, DiaRenderer *renderer)
 {
+  DiaRendererClass *renderer_ops = DIA_RENDERER_GET_CLASS (renderer);
   Element *elem;
   real w, h;
   Point points[4];
@@ -198,12 +205,12 @@ static void branch_draw(Branch *branch, Renderer *renderer)
   points[2].x = elem->corner.x + 2*w, points[2].y = elem->corner.y + h;
   points[3].x = elem->corner.x + w,   points[3].y = elem->corner.y + 2*h;
   
-  renderer->ops->set_fillstyle(renderer, FILLSTYLE_SOLID);
-  renderer->ops->set_linewidth(renderer, BRANCH_BORDERWIDTH);
-  renderer->ops->set_linestyle(renderer, LINESTYLE_SOLID);
+  renderer_ops->set_fillstyle(renderer, FILLSTYLE_SOLID);
+  renderer_ops->set_linewidth(renderer, BRANCH_BORDERWIDTH);
+  renderer_ops->set_linestyle(renderer, LINESTYLE_SOLID);
 
-  renderer->ops->fill_polygon(renderer, points, 4, &color_white);
-  renderer->ops->draw_polygon(renderer, points, 4, &color_black);
+  renderer_ops->fill_polygon(renderer, points, 4, &color_white);
+  renderer_ops->draw_polygon(renderer, points, 4, &color_black);
 }
 
 static void branch_update_data(Branch *branch)
@@ -217,12 +224,16 @@ static void branch_update_data(Branch *branch)
   /* Update connections: */
   branch->connections[0].pos.x = elem->corner.x;
   branch->connections[0].pos.y = elem->corner.y + elem->height / 2.;
+  branch->connections[0].directions = DIR_WEST;
   branch->connections[1].pos.x = elem->corner.x + elem->width / 2.;
   branch->connections[1].pos.y = elem->corner.y;
+  branch->connections[1].directions = DIR_NORTH;
   branch->connections[2].pos.x = elem->corner.x + elem->width;
   branch->connections[2].pos.y = elem->corner.y + elem->height / 2.;
+  branch->connections[2].directions = DIR_EAST;
   branch->connections[3].pos.x = elem->corner.x + elem->width / 2.;;
   branch->connections[3].pos.y = elem->corner.y + elem->height;
+  branch->connections[3].directions = DIR_SOUTH;
   
   element_update_boundingbox(elem);
   obj->position = elem->corner;
@@ -257,8 +268,12 @@ static Object *branch_create(Point *startpoint, void *user_data, Handle **handle
   elem->extra_spacing.border_trans = BRANCH_BORDERWIDTH / 2.0;
   branch_update_data(branch);
 
+  for (i=0;i<8;i++) {
+    obj->handles[i]->type = HANDLE_NON_MOVABLE;
+  }
+
   *handle1 = NULL;
-  *handle2 = obj->handles[0];
+  *handle2 = NULL;
   return &branch->element.object;
 }
 
@@ -267,62 +282,12 @@ static void branch_destroy(Branch *branch)
   element_destroy(&branch->element);
 }
 
-static Object *branch_copy(Branch *branch)
-{
-  int i;
-  Branch *newbranch;
-  Element *elem, *newelem;
-  Object *newobj;
-  
-  elem = &branch->element;
-  
-  newbranch = g_malloc0(sizeof(Branch));
-  newelem = &newbranch->element;
-  newobj = &newelem->object;
-
-  element_copy(elem, newelem);
-  for (i=0;i<8;i++)
-    {
-      newobj->connections[i] = &newbranch->connections[i];
-      newbranch->connections[i].object = newobj;
-      newbranch->connections[i].connected = NULL;
-      newbranch->connections[i].pos = branch->connections[i].pos;
-      newbranch->connections[i].last_pos = branch->connections[i].last_pos;
-    }
-  branch_update_data(newbranch);
-  return &newbranch->element.object;
-}
-
-static void branch_save(Branch *branch, ObjectNode obj_node, const char *filename)
-{
-  element_save(&branch->element, obj_node);
-}
-
 static Object *branch_load(ObjectNode obj_node, int version, const char *filename)
 {
-  Branch *branch;
-  Element *elem;
-  Object *obj;
-  int i;
-  
-  branch = g_malloc0(sizeof(Branch));
-  elem = &branch->element;
-  obj = &elem->object;
-  
-  obj->type = &branch_type;
-  obj->ops = &branch_ops;
-
-  element_load(elem, obj_node);
-
-  element_init(elem, 8, 8);
-
-  for (i=0;i<8;i++)
-    {
-      obj->connections[i] = &branch->connections[i];
-      branch->connections[i].object = obj;
-      branch->connections[i].connected = NULL;
-    }
-  elem->extra_spacing.border_trans = BRANCH_BORDERWIDTH / 2.0;
-  branch_update_data(branch);
-  return &branch->element.object;
+  return object_load_using_properties(&branch_type,
+                                      obj_node,version,filename);
 }
+
+
+
+
